//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "GPSCom2Client.h"


//////////////////// Implementation ///////////////////////////////////////////

class CMyClient : public GPSCom2::CClient
{
  void OnSentence(_In_ const GPSCom2::String& sSentence) override
  {
  #ifdef CGPSCOM2CLIENT_MFC_EXTENSIONS
    _tprintf(_T("Received sentence: %s\n"), sSentence.GetString());
  #else
    _tprintf(_T("Received sentence: %s\n"), sSentence.c_str());
  #endif //#ifdef CGPSCOM2CLIENT_MFC_EXTENSIONS

    //Let the base class do its thing
    __super::OnSentence(sSentence);
  }
};

int _tmain()
{
  //Initialize the COM subsystem
  HRESULT hr{CoInitialize(nullptr)};
  if (FAILED(hr))
    return 1;

  //Test out the NMEA parser with some sample sentences (which do not have fixes)
#ifdef _DEBUG
  GPSCom2::String sSentence{_T("$GPGGA,143413.48,,,,,0,00,99.99,,,,,,*6A")};
  GPSCom2::StringArray sFields;
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CGGASentence GGA;
  GGA.Parse(TRUE, sSentence, sFields);

  sSentence = _T("$GPVTG,,,,,,,,,N*30");
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CVTGSentence VTG;
  VTG.Parse(TRUE, sSentence, sFields);

  sSentence = _T("$GPGSA,A,1,,,,,,,,,,,,,99.99,99.99,99.99*30");
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CGSASentence GSA;
  GSA.Parse(TRUE, sSentence, sFields);

  sSentence = _T("$GPGSV,4,1,14,03,00,001,,05,47,180,,07,09,057,,08,45,058,*7C");
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CGSVSentence GSV;
  GSV.Parse(TRUE, sSentence, sFields);

  sSentence = _T("$GPGLL,,,,,143413.48,V,N*46");
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CGLLSentence GLL;
  GLL.Parse(TRUE, sSentence, sFields);

  sSentence = _T("$GPGST,143413.48,0.0000,,,,10,10,10*6A");
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CGSTSentence GST;
  GST.Parse(TRUE, sSentence, sFields);

  sSentence = _T("$GPRMC,143414.00,V,,,,,,,010510,,,N*7F");
  GPSCom2::CSentence::Tokenize(sSentence, sFields);
  GPSCom2::CRMCSentence RMC;
  RMC.Parse(TRUE, sSentence, sFields);
#endif //#ifdef _DEBUG

  //Create the client
  CMyClient client;
  hr = client.Open2(3); //You may want to change this value to match the serial port number which your GPS device is on on your machine
  if (SUCCEEDED(hr))
  {
    //Run the client for some time
    time_t nStartTime{0};
    time_t nCurrentTime{0};
    time(&nStartTime);
    MSG msg{};
    while (((nCurrentTime - nStartTime) < 60) && (GetMessage(&msg, nullptr, 0, 0) > 0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);

      //Prepare for the next loop
      time(&nCurrentTime);
    }

    //Close down the client
    client.Close();
  }
  else
    _tprintf(_T("Failed to open GPS port, please ensure the nPort parameter to the Open method is valid and the port is currently not open by some other program, Error:%08X\n"), hr);

  //Free up the COM subsystem
  CoUninitialize();

  return 0;
}
