/*
Module : GPSConnection.h
Purpose: Connection class for GPSCom2
Created: PJN / 27-04-2010

Copyright (c) 2010 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __GPSCONNECTION_H_
#define __GPSCONNECTION_H_


//////////////////// Includes /////////////////////////////////////////////////

#include "GPSCom2_i.h"
#include "_IGPSConnectionEvents_CP.H"


//////////////////// Classes //////////////////////////////////////////////////

class ATL_NO_VTABLE CGPSConnection :
  public ATL::CComObjectRootEx<ATL::CComSingleThreadModel>,
  public ATL::CComCoClass<CGPSConnection, &CLSID_GPSConnection>,
  public ATL::ISupportErrorInfoImpl<&IID_IGPSConnection>,
  public ATL::IDispatchImpl<IGPSConnection, &__uuidof(IGPSConnection)>,
  public ATL::IConnectionPointContainerImpl<CGPSConnection>,
  public ATL::IProvideClassInfo2Impl<&CLSID_GPSConnection, &__uuidof(_IGPSConnectionEvents)>,
  public CProxy_IGPSConnectionEvents<CGPSConnection>
{
public:
#pragma warning(suppress: 26440)
  DECLARE_NO_REGISTRY()

#pragma warning(suppress: 26440)
  BEGIN_CONNECTION_POINT_MAP(CGPSConnection) //NOLINT(modernize-avoid-c-arrays)
    CONNECTION_POINT_ENTRY(__uuidof(_IGPSConnectionEvents))
#pragma warning(suppress: 26485)
  END_CONNECTION_POINT_MAP()

#pragma warning(suppress: 26429 26440 26447 26477 26481)
  BEGIN_COM_MAP(CGPSConnection) //NOLINT(modernize-avoid-c-arrays, modernize-use-noexcept)
    COM_INTERFACE_ENTRY(IGPSConnection)
    COM_INTERFACE_ENTRY(IDispatch)
    COM_INTERFACE_ENTRY(ISupportErrorInfo)
    COM_INTERFACE_ENTRY(IConnectionPointContainer)
    COM_INTERFACE_ENTRY(IProvideClassInfo)
    COM_INTERFACE_ENTRY(IProvideClassInfo2)
#pragma warning(suppress: 26477 26485)
  END_COM_MAP()

#pragma warning(suppress: 26434)
  DECLARE_PROTECT_FINAL_CONSTRUCT()

  virtual void FinalRelease();

//Non COM related methods
  HRESULT Open(_In_ long nPort, _In_ BSTR bstrInitString);
  HRESULT Open(_In_ long nPort, _In_ long nDataBits, _In_ long nBaudRate, _In_ GPS_STOP_BITS StopBits, _In_ GPS_PARITY Parity, _In_ BSTR bstrInitString);
  void Close() noexcept;
  BOOL Write(_In_reads_bytes_opt_(dwSize) LPCVOID lpBuffer, _In_ DWORD dwSize, _In_ OVERLAPPED& overlapped) noexcept;
  static unsigned __stdcall _MonitorThread(_In_ void* pParameter);
  unsigned MonitorThread();
  void ReadData(_Inout_ ATL::CStringA& sBuffer, _Inout_ OVERLAPPED& overlapped);
  void HandleRead(_In_z_ LPCSTR pszNewData, _Inout_ ATL::CStringA& sBuffer);

//IGPSConnection
  STDMETHOD(GetSettings)(long* pnPort, long* pnDataBits, long* pnBaudRate, GPS_STOP_BITS* pStopBits, GPS_PARITY* pParity, BSTR* pbstrInitString);

//Member variables
  ATL::CHandle m_Port;
  ATL::CEvent m_StopEvent;
  ATL::CEvent m_DataEvent;
  long m_nDataBits{-1};
  long m_nBaudRate{-1};
  GPS_STOP_BITS m_StopBits{GPS_STOP_BITS_NONE};
  GPS_PARITY m_Parity{GPS_PARITY_NONE};
  long m_nPort{0};
  ATL::CComBSTR m_bstrInitString;
  ATL::CHandle m_MonitorThread;
};

#pragma warning(suppress: 26477)
OBJECT_ENTRY_AUTO(__uuidof(GPSConnection), CGPSConnection)


#endif //#ifndef __GPSCONNECTION_H_