/*
Module : GPSConnections.cpp
Purpose: Connections class for GPSCom2
Created: PJN / 27-04-2010

Copyright (c) 2010 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "GPSConnections.h"
#include "GPSConnection.h"
#include "GPSCom2.h"


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26433 26436)
STDMETHODIMP CGPSConnections::get_Count(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return Error(_T("Val is a null pointer"), IID_IGPSConnections, E_POINTER);

  //Synchronize access to the connections array
  ATL::CComCritSecLock<ATL::CComAutoCriticalSection> lock{_AtlModule.m_csConnections};
#pragma warning(suppress: 26472)
  *pVal = static_cast<long>(_AtlModule.m_Connections.GetCount());

  return S_OK;
}

#pragma warning(suppress: 26433 26487)
STDMETHODIMP CGPSConnections::Item(long nIndex, IGPSConnection** ppConnection) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (ppConnection == nullptr)
    return Error(_T("Connection is a null pointer"), IID_IGPSConnections, E_POINTER);

  //Initialize out value to null
  *ppConnection = nullptr;

  //Synchronize access to the connections array
  ATL::CComCritSecLock<ATL::CComAutoCriticalSection> lock{_AtlModule.m_csConnections};

  //Get the specified connection
#pragma warning(suppress: 26472)
  if ((nIndex >= 0) && (static_cast<size_t>(nIndex) < _AtlModule.m_Connections.GetCount()))
  {
    //Pull out the connection from the array
#pragma warning(suppress: 26429)
    ATL::CComObject<CGPSConnection>* pConnection{_AtlModule.m_Connections[nIndex]};
#pragma warning(suppress: 26477)
    ATLASSERT(pConnection != nullptr);

    //Query for the IGPSConnection interface for this array element
#pragma warning(suppress: 26490)
    return pConnection->QueryInterface(IID_IGPSConnections, reinterpret_cast<void**>(ppConnection));
  }
  else
  {
    //Create the rich error info
    return Error(_T("Invalid index value"), IID_IGPSConnections, E_INVALIDARG);
  }

  return S_OK;
}

#pragma warning(suppress: 26433 26487)
STDMETHODIMP CGPSConnections::get__NewEnum(IUnknown** ppVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (ppVal == nullptr)
    return Error(_T("Val is a null pointer"), IID_IGPSConnections, E_POINTER);

  //Set the Enumerator to null
  *ppVal = nullptr;

  //Create the enumerator instance
#pragma warning(suppress: 26429)
  ATL::CComObject<ATL::CComEnum<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT, ATL::_Copy<VARIANT>>>* pEnumerator{nullptr};
  HRESULT hr{ATL::CComObject<ATL::CComEnum<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT, ATL::_Copy<VARIANT>>>::CreateInstance(&pEnumerator)};
  if (FAILED(hr))
    return hr;

  //Synchronize access to the connections array
  ATL::CComCritSecLock<ATL::CComAutoCriticalSection> lock{_AtlModule.m_csConnections};

  //Allocate a VARIANT array which will be taken over by the enumerator
#pragma warning(suppress: 26472)
  long nConnections{static_cast<long>(_AtlModule.m_Connections.GetCount())};
  VARIANT* pVarData{nullptr};
#pragma warning(suppress: 26400 26409)
  ATLTRY(pVarData = new VARIANT[nConnections])
  if (pVarData == nullptr)
  {
    delete pEnumerator;
    return Error(_T("Failed to allocate VARIANT array"), IID_IGPSConnections, E_OUTOFMEMORY);
  }

  //Copy each item into the VARIANT array
  for (long i{0}; i<nConnections; i++)
  {
    //Initialize each variant
#pragma warning(suppress: 26481)
    VariantInit(&pVarData[i]);

    //Fill in each variant's IDispatch pointer
#pragma warning(suppress: 26481)
    pVarData[i].vt = VT_DISPATCH;
#pragma warning(suppress: 26481)
    pVarData[i].pdispVal = nullptr;
    ATL::CComPtr<IGPSConnection> pConnection;
    hr = Item(i, &pConnection);
    if (FAILED(hr))
    {
#pragma warning(suppress: 26401 26409)
      delete [] pVarData;
      delete pEnumerator;
      return hr;
    }

#pragma warning(suppress: 26481 26490)
    hr = pConnection->QueryInterface(IID_IDispatch, reinterpret_cast<void**>(&pVarData[i].pdispVal));
    if (FAILED(hr))
    {
#pragma warning(suppress: 26401 26409)
      delete [] pVarData;
      delete pEnumerator;
      return hr;
    }
  }

  //Initialize the enumerator
#pragma warning(suppress: 26477 26481)
  hr = pEnumerator->Init(&pVarData[0], &pVarData[nConnections], nullptr, ATL::AtlFlagTakeOwnership);
  if (FAILED(hr))
  {
#pragma warning(suppress: 26401 26409)
    delete [] pVarData;
    delete pEnumerator;
    return hr;
  }

#pragma warning(suppress: 26490)
  hr = pEnumerator->QueryInterface(IID_IUnknown, reinterpret_cast<void**>(ppVal));
  if (FAILED(hr))
  {
    delete pEnumerator;
    return hr;
  }

  return hr;
}
