/*
Module : GPSPort.cpp
Purpose: Port class for GPSCom2
Created: PJN / 27-04-2010

Copyright (c) 2010 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "GPSPort.h"
#include "GPSCom2.h"


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26433 26436 26487 26812)
STDMETHODIMP CGPSPort::CreateConnection(long Port, long BaudRate, GPS_PARITY parity, long DataBits, GPS_STOP_BITS StopBits, BSTR InitString, IGPSConnection** ppGPSConnection) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (ppGPSConnection == nullptr)
    return Error(L"GPSConnection is a null pointer", IID_IGPSPort, E_POINTER);

  //Synchronize access to the connections array
  ATL::CComCritSecLock<ATL::CComAutoCriticalSection> lock{_AtlModule.m_csConnections};

  //Try to find the requested port in the existing connections
  for (size_t i{0}; i<_AtlModule.m_Connections.GetCount(); i++)
  {
#pragma warning(suppress: 26429)
    ATL::CComObject<CGPSConnection>* pConnection{_AtlModule.m_Connections[i]};
#pragma warning(suppress: 26477)
    ATLASSERT(pConnection != nullptr);
    if (pConnection->m_nPort == Port)
    {
      //Query for the specific early bound interface which we want to hand back
#pragma warning(suppress: 26490)
      return pConnection->QueryInterface(IID_IGPSConnection, reinterpret_cast<void**>(ppGPSConnection));
    }
  }

  //We couldn't find an existing connection, then create a COM CGPSConnection object using an ATL creator
#pragma warning(suppress: 26429)
  ATL::CComObject<CGPSConnection>* pConnection{nullptr};
  HRESULT hr{ATL::CComObject<CGPSConnection>::CreateInstance(&pConnection)};
  if (SUCCEEDED(hr))
  {
    //Open up the connection
    hr = pConnection->Open(Port, DataBits, BaudRate, StopBits, parity, InitString);
    if (FAILED(hr))
      return hr;

    //Query for the specific early bound interface which we want to hand back
#pragma warning(suppress: 26490)
    hr = pConnection->QueryInterface(IID_IGPSConnection, reinterpret_cast<void**>(ppGPSConnection));

    //add to module array of shared instances
    if (SUCCEEDED(hr))
      _AtlModule.m_Connections.Add(pConnection);
  }

  return hr;
}

#pragma warning(suppress: 26433 26487)
STDMETHODIMP CGPSPort::CreateConnection2(long Port, BSTR InitString, IGPSConnection** ppGPSConnection) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (ppGPSConnection == nullptr)
    return Error(L"GPSConnection is a null pointer", IID_IGPSPort, E_POINTER);

  //Synchronize access to the connections array
  ATL::CComCritSecLock<ATL::CComAutoCriticalSection> lock{_AtlModule.m_csConnections};

  //Try to find the requested port in the existing connections
  for (size_t i{0}; i<_AtlModule.m_Connections.GetCount(); i++)
  {
#pragma warning(suppress: 26429)
    ATL::CComObject<CGPSConnection>* pConnection{_AtlModule.m_Connections[i]};
#pragma warning(suppress: 26477)
    ATLASSERT(pConnection != nullptr);
    if (pConnection->m_nPort == Port)
    {
      //Query for the specific early bound interface which we want to hand back
#pragma warning(suppress: 26490)
      return pConnection->QueryInterface(IID_IGPSConnection, reinterpret_cast<void**>(ppGPSConnection));
    }
  }

  //We couldn't find an existing connection, then create a COM CGPSConnection object using an ATL creator
#pragma warning(suppress: 26429)
  ATL::CComObject<CGPSConnection>* pConnection{nullptr};
  HRESULT hr{ATL::CComObject<CGPSConnection>::CreateInstance(&pConnection)};
  if (SUCCEEDED(hr))
  {
    //Open up the connection
    hr = pConnection->Open(Port, InitString);
    if (FAILED(hr))
      return hr;

    //Query for the specific early bound interface which we want to hand back
#pragma warning(suppress: 26490)
    hr = pConnection->QueryInterface(IID_IGPSConnection, reinterpret_cast<void**>(ppGPSConnection));

    //add to module array of shared instances
    if (SUCCEEDED(hr))
      _AtlModule.m_Connections.Add(pConnection);
  }

  return hr;
}
