/*
Module : _IGPSConnectionEvents_CP.h
Purpose: events handling class for GPSCom2
Created: PJN / 27-04-2010

Copyright (c) 2010 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once


//////////////////// Includes /////////////////////////////////////////////////

//Pull in the new improved IConnectionPoint implementation which works across threads by using the GIT
#include "ATLCPImplMT.h"


//////////////////// Classes //////////////////////////////////////////////////

template<class T>
class CProxy_IGPSConnectionEvents : public ATL::IConnectionPointImplMT<T, &__uuidof(_IGPSConnectionEvents), _IGPSConnectionEvents>
{
public:
#pragma warning(suppress: 26461)
  HRESULT Fire_OnSentence(_In_ BSTR Sentence)
  {
    ATL::CComVariant varResult{E_FAIL, VT_ERROR};
    ATL::CComVariant varParam;

    //Serialize access to the clients array of IConnectionPointImplMT which is accessed via GetInterfaceAt
    ATL::CComCritSecLock<ATL::CComAutoCriticalSection> lock{this->m_CPMTCritSec};

    const int nConnections{this->m_Clients.GetSize()};
    for (int nConnectionIndex{0}; nConnectionIndex<nConnections; nConnectionIndex++)
    {
      ATL::CComPtr<_IGPSConnectionEvents> sp;
      const HRESULT hr{this->GetInterfaceAt(nConnectionIndex, sp)};
      if (SUCCEEDED(hr))
      {
        VariantClear(&varResult);
        varParam = Sentence;
        DISPPARAMS disp{&varParam, nullptr, 1, 0};
        sp->Invoke(0x1, IID_NULL, LOCALE_USER_DEFAULT, DISPATCH_METHOD, &disp, &varResult, nullptr, nullptr);
      }
      else
        varResult = ATL::CComVariant{hr, VT_ERROR};
    }

    return varResult.scode;
  }
};
